@@ -4,18 +4,18 @@
 setMethod("Schur", signature(x = "dgeMatrix"),
           function(x, vectors = TRUE, ...) {
               if(length(x.x <- x@x) && !all(is.finite(range(x.x))))
-                  stop("'x' has non-finite values")
+                  stop(gettextf("'%s' has non-finite values", "x"), domain = NA)
               cl <- .Call(dgeMatrix_Schur, x, vectors, TRUE)
               if(all(cl$WI == 0)) {
                   vals <- cl$WR
                   T <- triu(cl$T)
               } else {
                   vals <- complex(real = cl$WR, imaginary = cl$WI)
-                  T <- .m2dense(cl$T, ".ge")
+                  T <- .m2dense(cl$T, ",ge")
               }
               if(vectors)
                   new("Schur", Dim = x@Dim, Dimnames = x@Dimnames,
-                      Q = .m2dense(cl$Z, ".ge"), T = T, EValues = vals)
+                      Q = .m2dense(cl$Z, ",ge"), T = T, EValues = vals)
               else list(T = T, EValues = vals)
           })
 
@@ -25,19 +25,18 @@ setMethod("Schur", signature(x = "dsyMatrix"),
               vals <- as.double(e$values)
               T <- new("ddiMatrix", Dim = x@Dim, x = vals)
               if(vectors)
-                  new("Schur", Dim = x@Dim, Dimnames = symmDN(x@Dimnames),
-                      Q = .m2dense(e$vectors, ".ge"), T = T, EValues = vals)
+                  new("Schur", Dim = x@Dim, Dimnames = symDN(x@Dimnames),
+                      Q = .m2dense(e$vectors, ",ge"), T = T, EValues = vals)
               else list(T = T, EValues = vals)
           })
 
 setMethod("Schur", signature(x = "matrix"),
           function(x, vectors = TRUE, ...) {
-              ## MJ: breaks package 'control' ?!
-              ## if(is.complex(x))
-              ##     stop("Schur(x) not yet supported for 'x' of type \"complex\"")
+              ## FIXME: wrong for complex, but package 'control' seems to
+              ##        rely on the complex->double coercion (!?)
               storage.mode(x) <- "double"
               if(length(x) && !all(is.finite(range(x))))
-                  stop("'x' has non-finite values")
+                  stop(gettextf("'%s' has non-finite values", "x"), domain = NA)
               cl <- .Call(dgeMatrix_Schur, x, vectors, FALSE)
               vals <-
                   if(all(cl$WI == 0))
@@ -51,39 +50,21 @@ setMethod("Schur", signature(x = "matrix"),
 ## FIXME: don't coerce from sparse to dense
 setMethod("Schur", signature(x = "generalMatrix"),
           function(x, vectors = TRUE, ...)
-              Schur(as(as(x, "dMatrix"), "unpackedMatrix"), vectors, ...))
+              Schur(.M2unpacked(.M2kind(x, ",")), vectors, ...))
 
 ## FIXME: don't coerce from sparse to dense
 setMethod("Schur", signature(x = "symmetricMatrix"),
           function(x, vectors = TRUE, ...)
-              Schur(as(as(x, "dMatrix"), "unpackedMatrix"), vectors, ...))
-
-setMethod("Schur", signature(x = "diagonalMatrix"),
-          function(x, vectors = TRUE, ...) {
-              d <- x@Dim
-              if(x@diag != "N") {
-                  vals <- rep.int(1, d[1L])
-                  T <- new("ddiMatrix", Dim = d, diag = "U")
-              } else {
-                  vals <- x@x
-                  if(length(vals) && !all(is.finite(range(vals))))
-                      stop("'x' has non-finite values")
-                  T <- new("ddiMatrix", Dim = d, x = vals)
-              }
-              if(vectors) {
-                  Q <- new("ddiMatrix", Dim = d, diag = "U")
-                  new("Schur", Dim = d, Dimnames = x@Dimnames,
-                      Q = Q, T = T, EValues = vals)
-              } else list(T = T, EValues = vals)
-          })
+              Schur(.M2unpacked(.M2kind(x, ",")), vectors, ...))
 
 setMethod("Schur", signature(x = "triangularMatrix"),
           function(x, vectors = TRUE, ...) {
+              x <- .M2kind(x, ",")
               n <- (d <- x@Dim)[1L]
               if(n == 0L)
                   x@uplo <- "U"
               else if(.M.kind(x) != "n" && !all(is.finite(range(x))))
-                  stop("'x' has non-finite values")
+                  stop(gettextf("'%s' has non-finite values", "x"), domain = NA)
               vals <- diag(x, names = FALSE)
               if(x@uplo == "U") {
                   if(vectors) {
@@ -103,6 +84,26 @@ setMethod("Schur", signature(x = "triangularMatrix"),
               }
           })
 
+setMethod("Schur", signature(x = "diagonalMatrix"),
+          function(x, vectors = TRUE, ...) {
+              x <- .M2kind(x, ",")
+              d <- x@Dim
+              if(x@diag != "N") {
+                  vals <- rep.int(1, d[1L])
+                  T <- new("ddiMatrix", Dim = d, diag = "U")
+              } else {
+                  vals <- x@x
+                  if(length(vals) && !all(is.finite(range(vals))))
+                      stop(gettextf("'%s' has non-finite values", "x"), domain = NA)
+                  T <- new("ddiMatrix", Dim = d, x = vals)
+              }
+              if(vectors) {
+                  Q <- new("ddiMatrix", Dim = d, diag = "U")
+                  new("Schur", Dim = d, Dimnames = x@Dimnames,
+                      Q = Q, T = T, EValues = vals)
+              } else list(T = T, EValues = vals)
+          })
+
 
 ## METHODS FOR CLASS: Schur
 ## ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
@@ -110,7 +111,9 @@ setMethod("Schur", signature(x = "triangularMatrix"),
 setMethod("expand1", signature(x = "Schur"),
           function(x, which, ...)
               switch(which, "Q" = x@Q, "T" = x@T, "Q." = t(x@Q),
-                     stop("'which' is not \"Q\", \"T\", or \"Q.\"")))
+                     stop(gettextf("'%1$s' is not \"%2$s\", \"%3$s\", or \"%2$s.\"",
+                                   "which", "Q", "T"),
+                          domain = NA)))
 
 setMethod("expand2", signature(x = "Schur"),
           function(x, ...) {
