@@ -162,7 +162,7 @@ cholmod_factor *sexp_as_cholmod_factor(cholmod_factor *L, SEXP from)
 	}
 
 	if (!cholmod_check_factor(L, &c))
-		error(_("'%s' failed"), "cholmod_check_factor");
+		error(_("'%s' failed in '%s'"), "cholmod_check_factor", __func__);
 	UNPROTECT(4);
 	return L;
 }
@@ -177,6 +177,10 @@ cholmod_factor *sexp_as_cholmod_factor(cholmod_factor *L, SEXP from)
  *
  * @param A a pointer to a cholmod_sparse struct, to be modified in-place.
  * @param from an S4 object inheriting from virtual class CsparseMatrix.
+ * @param checkUnit a boolean indicating if the unit diagonal of formally
+ *     unit triangular CsparseMatrix should be allocated.
+ * @param sortInPlace a boolean indicating if unsorted CsparseMatrix
+ *     should be sorted in place to avoid an allocation.
  *
  * @return A.
  */
@@ -202,19 +206,20 @@ cholmod_sparse *sexp_as_cholmod_sparse(cholmod_sparse *A, SEXP from,
 	int *pdim = INTEGER(dim), m = pdim[0], n = pdim[1];
 
 	SEXP p = PROTECT(GET_SLOT(from, Matrix_pSym)),
-		i = PROTECT(GET_SLOT(from, Matrix_iSym)),
-		cpi = PROTECT(checkpi(p, i, m, n));
+	     i = PROTECT(GET_SLOT(from, Matrix_iSym)),
+	   cpi = PROTECT(checkpi(p, i, m, n));
 	if (TYPEOF(cpi) != LGLSXP)
-		error(CHAR(STRING_ELT(cpi, 0)));
+		error(_("'%s' failed in '%s': %s"),
+		      "checkpi", __func__, CHAR(STRING_ELT(cpi, 0)));
 	int *pp = INTEGER(p), *pi = INTEGER(i), sorted = LOGICAL(cpi)[0];
-	R_xlen_t np = XLENGTH(p), ni = XLENGTH(i);
+	size_t np = (size_t) XLENGTH(p), ni = (size_t) XLENGTH(i);
 	if (!sorted && !sortInPlace) {
 		int *tmp;
-		tmp = (int *) R_alloc((size_t) np, sizeof(int));
-		memcpy(tmp, pp, (size_t) np * sizeof(int));
+		tmp = (int *) R_alloc(np, sizeof(int));
+		memcpy(tmp, pp, np * sizeof(int));
 		pp = tmp;
-		tmp = (int *) R_alloc((size_t) ni, sizeof(int));
-		memcpy(tmp, pi, (size_t) ni * sizeof(int));
+		tmp = (int *) R_alloc(ni, sizeof(int));
+		memcpy(tmp, pi, ni * sizeof(int));
 		pi = tmp;
 	}
 
@@ -222,7 +227,7 @@ cholmod_sparse *sexp_as_cholmod_sparse(cholmod_sparse *A, SEXP from,
 	A->ncol = n;
 	A->p = pp;
 	A->i = pi;
-	A->nzmax = (size_t) ni;
+	A->nzmax = ni;
 	A->stype = 0;
 	A->itype = CHOLMOD_INT;
 	A->xtype = CHOLMOD_PATTERN;
@@ -250,7 +255,7 @@ cholmod_sparse *sexp_as_cholmod_sparse(cholmod_sparse *A, SEXP from,
 		case 'i':
 		{
 			int *px = (TYPEOF(x) == LGLSXP) ? LOGICAL(x) : INTEGER(x);
-			double *rtmp = (double *) R_alloc(nx + 1, sizeof(double));
+			double *rtmp = (double *) R_alloc(nx, sizeof(double));
 			for (size_t ix = 0; ix < nx; ++ix)
 				rtmp[ix] = (px[ix] == NA_INTEGER)
 					? NA_REAL : (double) px[ix];
@@ -262,8 +267,8 @@ cholmod_sparse *sexp_as_cholmod_sparse(cholmod_sparse *A, SEXP from,
 		{
 			double *px = REAL(x);
 			if (!sorted && !sortInPlace) {
-				double *rtmp = (double *) R_alloc(nx + 1, sizeof(double));
-				memcpy(rtmp, px, (nx + 1) * sizeof(double));
+				double *rtmp = (double *) R_alloc(nx, sizeof(double));
+				memcpy(rtmp, px, nx * sizeof(double));
 				px = rtmp;
 			}
 			A->x = px;
@@ -274,8 +279,8 @@ cholmod_sparse *sexp_as_cholmod_sparse(cholmod_sparse *A, SEXP from,
 		{
 			Rcomplex *px = COMPLEX(x);
 			if (!sorted && !sortInPlace) {
-				Rcomplex *rtmp = (Rcomplex *) R_alloc(nx + 1, sizeof(Rcomplex));
-				memcpy(rtmp, px, (nx + 1) * sizeof(Rcomplex));
+				Rcomplex *rtmp = (Rcomplex *) R_alloc(nx, sizeof(Rcomplex));
+				memcpy(rtmp, px, nx * sizeof(Rcomplex));
 				px = rtmp;
 			}
 			A->x = px;
@@ -288,7 +293,7 @@ cholmod_sparse *sexp_as_cholmod_sparse(cholmod_sparse *A, SEXP from,
 		UNPROTECT(1); /* x */
 	}
 	if (!sorted && !cholmod_sort(A, &c))
-		error(_("'%s' failed"), "cholmod_sort");
+		error(_("'%s' failed in '%s'"), "cholmod_sort", __func__);
 	if (checkUnit && class[1] == 't' && n > 0) {
 		SEXP diag = GET_SLOT(from, Matrix_diagSym);
 		char di = *CHAR(STRING_ELT(diag, 0));
@@ -317,6 +322,135 @@ cholmod_sparse *sexp_as_cholmod_sparse(cholmod_sparse *A, SEXP from,
 	return A;
 }
 
+/**
+ * Coerce from TsparseMatrix to (cholmod_triplet *)
+ *
+ * Sets the members of a pointed-to cholmod_triplet struct, using "data"
+ * obtained from slots of a TsparseMatrix.  The result should _not_ be
+ * freed using cholmod_free_sparse, as the resulting members point to
+ * memory controlled by R, not by CHOLMOD.
+ *
+ * @param A a pointer to a cholmod_triplet struct, to be modified in-place.
+ * @param from an S4 object inheriting from virtual class TsparseMatrix.
+ * @param checkUnit a boolean indicating if the unit diagonal of formally
+ *     unit triangular TsparseMatrix should be allocated.
+ *
+ * @return A.
+ */
+cholmod_triplet *sexp_as_cholmod_triplet(cholmod_triplet *A, SEXP from,
+                                         Rboolean checkUnit)
+{
+	static const char *valid[] = {
+		"dgTMatrix", "dsTMatrix", "dtTMatrix",
+		"lgTMatrix", "lsTMatrix", "ltTMatrix",
+		"ngTMatrix", "nsTMatrix", "ntTMatrix", "" };
+	int ivalid = R_check_class_etc(from, valid);
+	if (ivalid < 0)
+		ERROR_INVALID_CLASS(from, __func__);
+	const char *class = valid[ivalid];
+	memset(A, 0, sizeof(cholmod_triplet));
+
+	SEXP dim = GET_SLOT(from, Matrix_DimSym);
+	int *pdim = INTEGER(dim), m = pdim[0], n = pdim[1];
+
+	SEXP i = PROTECT(GET_SLOT(from, Matrix_pSym)),
+		j = PROTECT(GET_SLOT(from, Matrix_iSym));
+	int *pi = INTEGER(i), *pj = INTEGER(j);
+	size_t nnz0 = (size_t) XLENGTH(i), nnz1 = nnz0;
+
+	if (checkUnit && class[1] == 't') {
+		SEXP diag = GET_SLOT(from, Matrix_diagSym);
+		char di = *CHAR(STRING_ELT(diag, 0));
+		if (di != 'N')
+			nnz1 += n;
+	}
+
+	if (nnz0 < nnz1) {
+		int *tmp;
+		tmp = (int *) R_alloc(nnz1, sizeof(int));
+		memcpy(tmp, pi, nnz1 * sizeof(int));
+		pi = tmp;
+		tmp = (int *) R_alloc(nnz1, sizeof(int));
+		memcpy(tmp, pj, nnz1 * sizeof(int));
+		pj = tmp;
+
+		pi += nnz0; pj += nnz0;
+		for (int d = 0; d < n; ++d)
+			*(pi++) = *(pj++) = d;
+		pi -= nnz1; pj -= nnz1;
+	}
+
+	A->nrow = m;
+	A->ncol = n;
+	A->i = pi;
+	A->j = pj;
+	A->nzmax = nnz1;
+	A->nnz = nnz1;
+	A->stype = 0;
+	A->itype = CHOLMOD_INT;
+	A->xtype = CHOLMOD_PATTERN;
+	A->dtype = CHOLMOD_DOUBLE;
+
+	if (class[1] == 's') {
+		SEXP uplo = GET_SLOT(from, Matrix_uploSym);
+		char ul = *CHAR(STRING_ELT(uplo, 0));
+		A->stype = (ul == 'U') ? 1 : -1;
+	}
+	if (class[0] != 'n') {
+		SEXP x = PROTECT(GET_SLOT(from, Matrix_xSym));
+		switch (class[0]) {
+		case 'l':
+		case 'i':
+		{
+			int *px = (TYPEOF(x) == LGLSXP) ? LOGICAL(x) : INTEGER(x);
+			double *rtmp = (double *) R_alloc(nnz1, sizeof(double));
+			for (size_t k =    0; k < nnz0; ++k)
+				rtmp[k] = (px[k] == NA_INTEGER)
+					? NA_REAL : (double) px[k];
+			for (size_t k = nnz0; k < nnz1; ++k)
+				rtmp[k] = 1.0;
+			A->x = rtmp;
+			A->xtype = CHOLMOD_REAL;
+			break;
+		}
+		case 'd':
+		{
+			double *px = REAL(x);
+			if (nnz0 < nnz1) {
+				double *rtmp = (double *) R_alloc(nnz1, sizeof(double));
+				memcpy(rtmp, px, nnz0 * sizeof(double));
+				for (size_t k = nnz0; k < nnz1; ++k)
+					rtmp[k] = 1.0;
+				px = rtmp;
+			}
+			A->x = px;
+			A->xtype = CHOLMOD_REAL;
+			break;
+		}
+		case 'z':
+		{
+			Rcomplex *px = COMPLEX(x);
+			if (nnz0 < nnz1) {
+				Rcomplex *rtmp = (Rcomplex *) R_alloc(nnz1, sizeof(Rcomplex));
+				memcpy(rtmp, px, nnz0 * sizeof(Rcomplex));
+				for (size_t k = nnz0; k < nnz1; ++k)
+					rtmp[k] = Matrix_zone;
+				px = rtmp;
+			}
+			A->x = px;
+			A->xtype = CHOLMOD_COMPLEX;
+			break;
+		}
+		default:
+			break;
+		}
+		UNPROTECT(1); /* x */
+	}
+
+	UNPROTECT(2); /* j, i */
+	return A;
+}
+
 /**
  * Coerce from [nlidz]geMatrix or vector to (cholmod_dense *)
  *
@@ -677,6 +811,114 @@ SEXP cholmod_sparse_as_sexp(cholmod_sparse *A, int doFree,
 	return to;
 }
 
+/**
+ * Coerce from (cholmod_triplet *) to TsparseMatrix
+ *
+ * Allocates an S4 object inheriting from virtual class TsparseMatrix
+ * and copies into the slots from members of a pointed-to cholmod_triplet
+ * struct.  The specific class of the result is determined by struct
+ * members xtype and stype and by arguments ttype and doLogic.
+ *
+ * @param A a pointer to a cholmod_triplet struct.
+ * @param doFree a flag indicating if and how to free A before returning.
+ *     (0) don't free, (>0) free with cholmod_free_triplet, (<0) free with
+ *     R_Free.
+ * @param ttype a flag indicating if the result should be a .tTMatrix.
+ *     (0) not .tTMatrix, (>0) .tTMatrix with uplo="U", (<0) .tTMatrix
+ *     with uplo="L".  If ttype=0, then the result is a .gTMatrix or
+ *     .sTMatrix depending on stype.  (0) .gTMatrix, (>0) .sTMatrix with
+ *     uplo="U", (<0) .sTMatrix with uplo="L".
+ * @param doLogic a flag indicating if the result should be an l.TMatrix
+ *     if xtype=CHOLMOD_REAL.
+ * @param diagString a null-terminated string or NULL.  The diag slot
+ *     of a .tTMatrix result is "N" if and only if diagString is NULL
+ *     or diagString[0] is 'N'.
+ * @param dimnames an R object specifying the Dimnames slot of the result,
+ *     unused if not a list of length 2.
+ *
+ * @return A TsparseMatrix.
+ */
+SEXP cholmod_triplet_as_sexp(cholmod_triplet *A, int doFree,
+                             int ttype, int doLogic, const char *diagString,
+                             SEXP dimnames)
+{
+
+#define FREE_THEN(_EXPR_) \
+	do { \
+		if (doFree != 0) { \
+			if (doFree < 0) \
+				R_Free(A); \
+			else if (A->itype == CHOLMOD_INT) \
+				cholmod_free_triplet(&A, &c); \
+			else \
+				cholmod_l_free_triplet(&A, &cl); \
+			_EXPR_; \
+		} \
+	} while (0)
+
+	if (A->itype != CHOLMOD_INT)
+		FREE_THEN(error(_("wrong '%s'"), "itype"));
+	if (A->xtype != CHOLMOD_PATTERN &&
+	    A->xtype != CHOLMOD_REAL && A->xtype != CHOLMOD_COMPLEX)
+		FREE_THEN(error(_("wrong '%s'"), "xtype"));
+	if (A->dtype != CHOLMOD_DOUBLE)
+		FREE_THEN(error(_("wrong '%s'"), "dtype"));
+	if (A->nrow > INT_MAX || A->ncol > INT_MAX)
+		FREE_THEN(error(_("dimensions cannot exceed %s"), "2^31-1"));
+	int m = (int) A->nrow, n = (int) A->ncol;
+	R_xlen_t nnz = (R_xlen_t) A->nnz;
+	char class[] = "..TMatrix";
+	class[0] = (A->xtype == CHOLMOD_PATTERN)
+		? 'n' : ((A->xtype == CHOLMOD_COMPLEX) ? 'z' : ((doLogic) ? 'l' : 'd'));
+	class[1] = (ttype != 0) ? 't' : ((A->stype != 0) ? 's' : 'g');
+	SEXP to = PROTECT(newObject(class)),
+		dim = PROTECT(GET_SLOT(to, Matrix_DimSym)),
+		i = PROTECT(allocVector(INTSXP, nnz)),
+		j = PROTECT(allocVector(INTSXP, nnz));
+	INTEGER(dim)[0] = m;
+	INTEGER(dim)[1] = n;
+	memcpy(INTEGER(i), A->i, (size_t) nnz * sizeof(int));
+	memcpy(INTEGER(j), A->j, (size_t) nnz * sizeof(int));
+	SET_SLOT(to, Matrix_iSym, i);
+	SET_SLOT(to, Matrix_jSym, j);
+	if (A->xtype != CHOLMOD_PATTERN) {
+		SEXP x;
+		if (A->xtype == CHOLMOD_COMPLEX) {
+			PROTECT(x = allocVector(CPLXSXP, nnz));
+			memcpy(COMPLEX(x), A->x, (size_t) nnz * sizeof(Rcomplex));
+		} else if (!doLogic) {
+			PROTECT(x = allocVector(REALSXP, nnz));
+			memcpy(REAL(x), A->x, (size_t) nnz * sizeof(double));
+		} else {
+			PROTECT(x = allocVector(LGLSXP, nnz));
+			int k, *px = LOGICAL(x);
+			double *py = (double *) A->x;
+			for (k = 0; k < nnz; ++k)
+				px[k] = (ISNAN(py[k])) ? NA_LOGICAL : (py[k] != 0.0);
+		}
+		SET_SLOT(to, Matrix_xSym, x);
+		UNPROTECT(1);
+	}
+	if (ttype < 0 || A->stype < 0) {
+		SEXP uplo = PROTECT(mkString("L"));
+		SET_SLOT(to, Matrix_uploSym, uplo);
+		UNPROTECT(1);
+	}
+	if (ttype != 0 && diagString && diagString[0] != 'N') {
+		SEXP diag = PROTECT(mkString("U"));
+		SET_SLOT(to, Matrix_diagSym, diag);
+		UNPROTECT(1);
+	}
+	if (TYPEOF(dimnames) == VECSXP && LENGTH(dimnames) == 2)
+		SET_SLOT(to, Matrix_DimNamesSym, dimnames);
+	FREE_THEN();
+
+#undef FREE_THEN
+
+	UNPROTECT(4);
+	return to;
+}
+
 /**
  * Coerce from (cholmod_dense *) to [dz]geMatrix
  *
@@ -810,10 +1052,10 @@ cholmod_factor *cholmod_factor_update(cholmod_factor *L, cholmod_sparse *A,
 	z[0] = beta;
 	z[1] = 0.0;
 	if (!cholmod_factorize_p(A, z, NULL, 0, L, &c))
-		error(_("'%s' failed"), "cholmod_factorize_p");
+		error(_("'%s' failed in '%s'"), "cholmod_factorize_p", __func__);
 	if (L->is_ll != ll &&
 	    !cholmod_change_factor(L->xtype, ll, L->is_super, 1, 1, L, &c))
-		error(_("'%s' failed"), "cholmod_change_factor");
+		error(_("'%s' failed in '%s'"), "cholmod_change_factor", __func__);
 	return L;
 }
 
@@ -846,7 +1088,7 @@ int R_cholmod_start(cholmod_common *Common)
 {
 	int ans = cholmod_start(Common);
 	if (!ans)
-		error(_("'%s' failed"), "cholmod_start");
+		error(_("'%s' failed in '%s'"), "cholmod_start", __func__);
 #if 0
 	/* No longer, with SuiteSparse 5.7.1 : */
 	Common->print_function =
@@ -864,7 +1106,7 @@ int R_cholmod_finish(cholmod_common *Common)
 {
 	int ans = cholmod_finish(Common);
 	if (!ans)
-		error(_("'%s' failed"), "cholmod_finish");
+		error(_("'%s' failed in '%s'"), "cholmod_finish", __func__);
 	return ans;
 }
 
